@@ -8,7 +8,7 @@ package ISG::ParseConfig;
 use strict;
 
 use vars qw($VERSION);
-$VERSION = 1.5;
+$VERSION = 1.6;
 
 sub new($$)
 {
@@ -407,30 +407,26 @@ sub _parse_table($$)
 
 sub _parse_text($$)
 {
-    my $self = shift;
-    local $_ = shift;
+    my ($self, $line) = @_;
 
-    my $g = $self->{grammar}{_text};
-
-    if (defined $self->{cfg}{_text}) {
-        $self->{cfg}{_text} .= "\n";
-    }
-    else {
-        $self->{cfg}{_text} = '';
-    }
-    $self->{cfg}{_text} .= $_;
+    $self->{cfg}{_text} .= $line;
 
     return 1;
 }
 
 sub _check_text($$)
 {
-    my $self = shift;
-    my $name = shift;
+    my ($self, $name) = @_;
 
     my $g = $self->{grammar}{_text};
     defined $g or return 1;
 
+    # chop empty lines at beginning and end
+    if(defined $self->{cfg}{_text}) {
+	$self->{cfg}{_text} =~ s/\A([ \t]*[\n\r]+)*//m;
+	$self->{cfg}{_text} =~  s/^([ \t]*[\n\r]+)*\Z//m;
+    }
+
     # TODO: not good for META. Use _mandatory ?
     #defined $self->{cfg}{_text} or do {
     #  $self->_make_error("value of '$name' not defined");
@@ -460,10 +456,12 @@ sub _check_text($$)
 
 sub _parse_file($$);
 
-sub _parse_line($$)
+sub _parse_line($$$)
 {
     my $self = shift;
     local $_ = shift;
+    my $source = shift;
+
     /^\@include\s+["']?(.*)["']?$/ and do {
         push @{$self->{file_stack}}, $self->{file};
         push @{$self->{line_stack}}, $self->{line};
@@ -494,7 +492,7 @@ sub _parse_line($$)
     };
 
     if (defined $self->{grammar}{_text}) {
-        $self->_parse_text($_) or return 0;
+        $self->_parse_text($source) or return 0;
         return 1;
     }
     /^(\S+)\s*=\s*(.*)$/ and do {
@@ -527,11 +525,13 @@ sub _parse_file($$)
     $self->{file} = $file;
 
     local $_;
+    my $source = '';
     while (<File>) {
+	$source .= $_;
         chomp;
         s/^\s+//;
         s/\s+$//;            # trim
-        s/\s*#.*$//;            # comments
+        s/\s*#.*$//;         # comments
         next if $_ eq '';    # empty lines
         while (/\\$/) {# continuation
             s/\\$//;
@@ -544,7 +544,8 @@ sub _parse_file($$)
         }
 
         $self->{line} = $.;
-        $self->_parse_line($_) or do{ close File; return 0; };
+        $self->_parse_line($_, $source) or do{ close File; return 0; };
+	$source = '';
     }
     close File;
     return 1;
@@ -1070,6 +1071,7 @@ S<Tobias Oetiker E<lt>oetiker@ee.ethz.chE<gt>>
  2002-01-28 to     Fixed quote parsing in tables
  2002-03-12 ds 1.5 Implemented @define, make makepod return a string and not an array
  2002-08-28 to     Added maketmpl methode
+ 2002-10-10 ds 1.6 More verbatim _text sections
 
 =cut
 
