@@ -17,7 +17,7 @@ use File::Basename;
 
 # globale persistent variables for speedy
 use vars qw($cfg $probes $VERSION $havegetaddrinfo);
-$VERSION="1.16";
+$VERSION="1.24";
 
 # we want opts everywhere
 my %opt;
@@ -38,7 +38,7 @@ sub load_probes ($){
         die "$@\n" if $@;
 	eval '$prbs{$probe} = probes::'.$probe.'->new( $cfg->{Probes}{${probe}},$cfg );';
         die "$@\n" if $@;
-        die "Faild to load Probe $probe\n" unless defined $prbs{$probe};
+        die "Failed to load Probe $probe\n" unless defined $prbs{$probe};
     }
     return \%prbs;
 };
@@ -97,6 +97,7 @@ sub update_dynaddr ($$){
 sub sendmail ($$$){
     my $from = shift;
     my $to = shift;
+    $to = $1 if $to =~ /<(.*?)>/;
     my $body = shift;
     if ($cfg->{General}{mailhost}){
 	my $smtp = Net::SMTP->new($cfg->{General}{mailhost});
@@ -115,8 +116,8 @@ sub sendmail ($$$){
 
 sub init_alerts ($){
     my $cfg = shift;
-    foreach my $a (keys %{$cfg->{Alerts}}) {
-	my $x = $cfg->{Alerts}{$a};
+    foreach my $al (keys %{$cfg->{Alerts}}) {
+	my $x = $cfg->{Alerts}{$al};
         next unless ref $x eq 'HASH';
 	my $sub = " sub { my \$d = shift; my \$y = \$d->{$x->{type}};\n";
 	my @ops = split /\s*,\s*/, $x->{pattern};
@@ -124,8 +125,8 @@ sub init_alerts ($){
         my $i = - $x->{length};
         $sub .= "return 0 unless $x->{length} <= scalar \@\$y ;\n";
 	for (@ops) {
-	    /^(==|!=|<|>|<=|>=)(\d+|U|\*)(%?)$/
-		or die "ERROR Alert $a pattern entry '$_' is invalid\n";
+	    /^(==|!=|<|>|<=|>=)(\d+(?:\.\d*)?|U|\*)(%?)$/
+		or die "ERROR Alert $al pattern entry '$_' is invalid\n";
 	    my $op = $1;
 	    my $value = $2;
             my $perc = $3;
@@ -135,15 +136,15 @@ sub init_alerts ($){
 		} elsif ($op eq '!=') {
 		    $sub .= "return 0 unless defined \$y->[$i];\n";
 		} else {
-		    die "ERROR: invalid operator $op in connection U in Alert $a definition\n";
+		    die "ERROR: invalid operator $op in connection U in Alert $al definition\n";
 		}
 	    } elsif ($value eq '*') {
 		if ($op ne '==') {
-		    die "ERROR: operator $op makes no sense with * in Alert $a definition\n";
+		    die "ERROR: operator $op makes no sense with * in Alert $al definition\n";
 		} # do nothing else ...
 	    } else {
 		if ( $x->{type} eq 'loss') {
-                    die "ERROR: loss should be specified in % (alert $a pattern)\n" unless $perc eq "%";
+                    die "ERROR: loss should be specified in % (alert $al pattern)\n" unless $perc eq "%";
 		    $value = $value / 100 * $cfg->{Database}{pings};
 		} elsif ( $x->{type} eq 'rtt' ) {
 		    $value /= 1000;
@@ -157,7 +158,7 @@ sub init_alerts ($){
         $sub .= "return 1; }";
 	warn "### Compiling\n$sub\n" if $opt{debug};
 	$x->{sub} = eval ( $sub );
-	die "ERROR: compiling alert pattern $a ($x->{pattern}: $@\n" if $@;
+	die "ERROR: compiling alert pattern $al ($x->{pattern}: $@\n" if $@;
     }
 }
 
@@ -182,20 +183,20 @@ sub init_target_tree ($$$$$$$) {
     };
     
     $tree->{alerts} = $alerts
-	if (not defined $tree->{alerts} and defined $alerts);
+	if not defined $tree->{alerts} and defined $alerts;
 
     $alerts = $tree->{alerts};
 
     if ($tree->{alerts}){
 	die "ERROR: no Alerts section\n"
 	    unless exists $cfg->{Alerts};
-	$tree->{alerts} = [ split /\s*,\s*/, $tree->{alerts} ];
+	$tree->{alerts} = [ split(/\s*,\s*/, $tree->{alerts}) ];
 	$tree->{fetchlength} = 0;
- 	foreach $a (@{$tree->{alerts}}) {
-	    die "ERROR: alert $a ($name) is not defined"
-		unless defined $cfg->{Alerts}{$a};
-	    $tree->{fetchlength} = $cfg->{Alerts}{$a}{length}
-		if $tree->{fetchlength} < $cfg->{Alerts}{$a}{length};
+ 	foreach my $al (@{$tree->{alerts}}) {
+	    die "ERROR: alert $al ($name) is not defined"
+		unless defined $cfg->{Alerts}{$al};
+	    $tree->{fetchlength} = $cfg->{Alerts}{$al}{length}
+		if $tree->{fetchlength} < $cfg->{Alerts}{$al}{length};
 	}
     };
 
@@ -211,14 +212,31 @@ sub init_target_tree ($$$$$$$) {
 	    };
 	    init_target_tree $cfg, $probes, $probe, $tree->{$prop}, "$name/$prop", $PROBE_CONF,$alerts;
 	}
+        # if there is a filter do neither schedule these nor make rrds
+        if ($opt{filter} && scalar @{$opt{filter}}){
+            my $ok = 0;
+            for (@{$opt{filter}}){
+              /^\!(.+)$/ && do {
+                "$name" =~ /^$1/ && do{ $ok = 0};
+                next;
+              };
+              /^(.+)$/ && do {
+                "$name" =~ /^$1/ && do {$ok = 1};
+                next;
+              }; 
+            }  
+            next unless $ok; 
+        };
+
 	if ($prop eq 'host') {
 	    if (not -f $name.".rrd"){
 		RRDs::create
 			($name.".rrd", "--step",$cfg->{Database}{step},
 			      "DS:uptime:GAUGE:".(2*($cfg->{Database}{step})).":0:U",
 			      "DS:loss:GAUGE:".(2*($cfg->{Database}{step})).":0:".($cfg->{Database}{pings}),
-			      "DS:median:GAUGE:".(2*($cfg->{Database}{step})).":0:10",
-			      (map { "DS:ping${_}:GAUGE:".(2*($cfg->{Database}{step})).":0:10" }
+                               # 180 Seconds  is the max rtt we consider valid ... 
+			      "DS:median:GAUGE:".(2*($cfg->{Database}{step})).":0:180",
+			      (map { "DS:ping${_}:GAUGE:".(2*($cfg->{Database}{step})).":0:180" }
 			                                                  1..($cfg->{Database}{pings})),
 			      (map { "RRA:".(join ":", @{$_}) } @{$cfg->{Database}{_table}} ));
 		my $ERROR = RRDs::error();
@@ -311,8 +329,10 @@ sub target_menu($$$){
     my $current =  shift @{$open} || "";
      
     my @hashes;
-    foreach my $prop (sort {$a cmp $b} keys %{$tree}) {
-	push @hashes, $prop if ref $tree->{$prop} eq 'HASH' and $prop ne "PROBE_CONF";
+    foreach my $prop (sort { $tree->{$a}{_order} <=> $tree->{$b}{_order}}
+                      grep { ref $tree->{$_} eq 'HASH' and $_ ne "PROBE_CONF" }
+                      keys %{$tree}) {
+	push @hashes, $prop;
     }
     return "" unless @hashes;
     $print .= "<table width=\"100%\" class=\"menu\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
@@ -388,10 +408,9 @@ sub get_overview ($$$$){
     my $date = $cfg->{Presentation}{overview}{strftime} ? 
         POSIX::strftime($cfg->{Presentation}{overview}{strftime},
                         localtime(time)) : scalar localtime(time);
-    foreach my $prop (sort {$a cmp $b} keys %$tree) {
-	next unless ref $tree->{$prop} eq 'HASH';
-	next if $prop eq "PROBE_CONF";
-	next unless defined $tree->{$prop}{host};
+    foreach my $prop (sort {$tree->{$a}{_order} <=> $tree->{$b}{_order}} 
+                      grep {  ref $tree->{$_} eq 'HASH' and $_ ne "PROBE_CONF" and defined $tree->{$_}{host}}
+                      keys %$tree) {
         my $rrd = $cfg->{General}{datadir}.$dir."/$prop.rrd";
         my $max =  $cfg->{Presentation}{overview}{max_rtt} || "100000";
         my $medc = $cfg->{Presentation}{overview}{median_color} || "ff0000";
@@ -520,8 +539,10 @@ sub get_detail ($$$$){
 	
     }
     my $rrd = $cfg->{General}{datadir}."/".(join "/", @dirs)."/${file}.rrd";
+    my $img = $cfg->{General}{imgcache}."/".(join "/", @dirs)."/${file}.rrd";
+
     my %lasthight;
-    if (open (HG,"<${rrd}.maxhight")){
+    if (open (HG,"<${img}.maxhight")){
         while (<HG>){
           chomp;
           my @l = split / /;
@@ -530,7 +551,7 @@ sub get_detail ($$$$){
         close HG;
     }
     my $max = findmax $cfg, $rrd;
-    if (open (HG,">${rrd}.maxhight")){
+    if (open (HG,">${img}.maxhight")){
         foreach my $s (keys %{$max}){
           print HG "$s $max->{$s}\n";        
         }
@@ -554,7 +575,7 @@ sub get_detail ($$$$){
 	@median = ("DEF:median=${rrd}:median:AVERAGE",
 		   "DEF:loss=${rrd}:loss:AVERAGE",
                    "CDEF:ploss=loss,$cfg->{Database}{pings},/,100,*",
-           	   "GPRINT:median:AVERAGE:Median Ping RTT (avg %.1lf %ss) (",
+           	   "GPRINT:median:AVERAGE:Median Ping RTT (avg %.1lf %ss) ",
                    "LINE1:median#202020"
         	   );
 	my $p = $cfg->{Database}{pings}; 
@@ -590,7 +611,7 @@ sub get_detail ($$$$){
 	     );
              $last = $loss;
 	}
-	push @median, ( "GPRINT:ploss:AVERAGE:avg pkg loss\\: %.2lf %%\\l" );
+	push @median, ( "GPRINT:ploss:AVERAGE:)  avg pkg loss\\: %.2lf %%\\l" );
 #	map {print "$_<br/>"} @median;
     };
         # if we have uptime draw a colorful background or the graph showing the uptime
@@ -751,8 +772,8 @@ sub update_rrds($$$$$) {
             my $ERROR = RRDs::error();
 	    die "ERROR: $ERROR\n" if $ERROR;
 	    # check alerts
-# disabled
-	    if ( $tree->{alerts}) {
+            # disabled
+	    if ( $tree->{alerts} ) {
                 $tree->{stack} = {loss=>[],rtt=>[]} unless defined $tree->{stack};
 		my $x = $tree->{stack};
 		my ($loss,$rtt) = 
@@ -766,6 +787,14 @@ sub update_rrds($$$$$) {
 		    shift @{$x->{rtt}};
 		}
 		for (@{$tree->{alerts}}) {
+                    if ( not $cfg->{Alerts}{$_} ) {
+                        warn "WARNING: Empty alert in ".(join ",", @{$tree->{alerts}})." ($name)\n";
+                        next;
+                    };
+                    if ( ref $cfg->{Alerts}{$_}{sub} ne 'CODE' ) {
+       		        warn "WARNING: Alert '$_' did not resolve to s Sub Ref. Skipping\n";
+                        next;
+                    };
 		    if ( &{$cfg->{Alerts}{$_}{sub}}($x) ){
 			# we got a match
 			my $from;
@@ -773,6 +802,9 @@ sub update_rrds($$$$$) {
                         my $base = $cfg->{General}{datadir};
                         $line =~ s|^$base/||;
                         $line =~ s|/host$||;
+                        $line =~ s|/|.|g;
+                        my $urlline = $line;
+                        $urlline =  $cfg->{General}{cgiurl}."?target=".$line;
                         my $loss = "loss: ".join ", ",map {sprintf "%.0f %%", ( $_ || 0 ) / $cfg->{Database}{pings} * 100} @{$x->{loss}};
                         my $rtt = "rtt: ".join ", ",map {$_ ? (sprintf "%.0f ms", $_*1000):"U" } @{$x->{rtt}}; 
                         my $stamp = scalar localtime time;
@@ -786,7 +818,7 @@ Subject: [SmokeAlert] $_ on $line
 
 $stamp
 
-Got a match for alert "$_" for $line
+Got a match for alert "$_" for $urlline
 
 Pattern
 -------
@@ -818,6 +850,7 @@ sub get_parser () {
       {
        _sections => [ ( "PROBE_CONF", "/$KEY_RE/" ) ],
        _vars     => [ qw (probe menu title alerts note email host remark) ],
+       _order    => 1,
        _mandatory=> [ qw (menu title) ],
        _doc => <<DOC,
 Each target section can contain information about a host to monitor as
@@ -1340,10 +1373,10 @@ coloring
 
 Example:
 
- # Uptime Color            Legend
- 3600    00ff00   "<1h"
- 86400   0000ff   "<1d"
- 604800  ff0000   "<1w"
+ # Uptime      Color     Legend
+ 3600          00ff00   "<1h"
+ 86400         0000ff   "<1d"
+ 604800        ff0000   "<1w"
  1000000000000 ffff00   ">1w"
 
 Uptime is in days!
@@ -1409,8 +1442,8 @@ alert should be sent at all.
 The patterns are numbers prefixed with an operator indicating the type
 of comparison required for a match.
 
-The following RTT pattern detects if a targets RTT goes from consnt
-below 10ms to constant 100ms and more:
+The following RTT pattern detects if a targets RTT goes from constanty
+below 10ms to constantly 100ms and more:
 
  old ------------------------------> new
  <10,<10,<10,<10,<10,>10,>100,>100,>100
@@ -1433,11 +1466,13 @@ A complete example
 
  +lossdetect
  type = loss
+ # in percent
  pattern = ==0%,==0%,==0%,==0%,>20%,>20%,>20%
  comment = suddenly there is packet loss
 
  +rttdetect
  type = rtt
+ # in milli seconds
  pattern = <10,<10,<10,<10,<10,<100,>100,>100,>100
  comment = routing mesed up again ?
 
@@ -1466,8 +1501,8 @@ using the following 5 arguments: B<name-of-alert>, B<target>, B<loss-pattern>, B
                      _re_error => 'Use loss or rtt'
 			  },
    	 	  pattern => {
- 		     _doc => "a comma separated list of comparison operators and numbers",
-		     _re => '(==|<|<=|>|>=|!=)(\d+|U|\*)%?(,(?:==|<|<=|>|>=|!=)(?:\d+|U|\*)%?)*',
+ 		     _doc => "a comma separated list of comparison operators and numbers. rtt patterns are in milli seconds, loss patterns are in percent",
+		     _re => '(==|<|<=|>|>=|!=)(\d+(?:\.\d*)?|U|\*)%?(,(?:==|<|<=|>|>=|!=)(?:\d+(?:\.\d*)?|U|\*)%?)*',
  		     _re_error => 'Could not parse pattern',
 		             },
 		  },
@@ -1479,6 +1514,7 @@ Each section can contain one host as well as other sections.
 DOC
 		   _vars       => [ qw(probe menu title remark alerts) ],
 		   _mandatory  => [ qw(probe menu title) ],
+                   _order => 1,
 		   _sections   => [ ( "PROBE_CONF", "/$KEY_RE/" ) ],
 		   probe => { _doc => <<DOC },
 The name of the probe module to be used for this host. The value of
@@ -1562,7 +1598,7 @@ sub demonize_me ($) {
         &POSIX::setsid or die "Can't start a new session: $!";
         open STDOUT,'>/dev/null' or die "ERROR: Redirecting STDOUT to /dev/null: $!";
         open STDIN, '</dev/null' or die "ERROR: Redirecting STDIN from /dev/null: $!";
-        open STDERR, '</dev/null' or die "ERROR: Redirecting STDERR from /dev/null: $!";
+        open STDERR, '>/dev/null' or die "ERROR: Redirecting STDERR to /dev/null: $!";
     }
 }
 
@@ -1625,7 +1661,7 @@ my $RCS_VERSION = '$Id: Smokeping.pm,v 1.4 2002/03/09 12:01:08 oetiker Exp $';
 
 sub load_cfg ($) { 
     my $cfgfile = shift;
-    my $cfmod = (stat $cfgfile)[9];
+    my $cfmod = (stat $cfgfile)[9] || die "ERROR: calling stat on $cfgfile: $!\n";
     # when running under speedy this will prevent reloading on every run
     # if cfgfile has been modified we will still run.
     if (not defined $cfg or $cfg->{__last} < $cfmod ){
@@ -1677,7 +1713,7 @@ POD
 
 ${e}head1 COPYRIGHT
 
-Copyright (c) 2001 by Tobias Oetiker. All right reserved.
+Copyright (c) 2001-2003 by Tobias Oetiker. All right reserved.
 
 ${e}head1 LICENSE
 
@@ -1738,6 +1774,7 @@ sub cgi ($) {
 sub main ($) {
     umask 022;
     my $cfgfile = shift;
+    $opt{filter}=[];
     GetOptions(\%opt, 'version', 'email', ,'man','help',
                       'makepod','debug','restart', 'nodemon|nodaemon') or pod2usage(2);
     if($opt{version})  { print "$RCS_VERSION\n"; exit(0) };
